#include <avr/io.h>
#include <avr/pgmspace.h>
#include <util/delay.h>

#include "message.h"

#define WPM 15			// speed in words-per-minute
#define DIT_TIME (1200000/WPM)	// dit interval in microseconds

#define BEEP_TONE 800				// Hz
#define BEEP_HALF_PERIOD 1000000/BEEP_TONE	// us

#define KEYL _BV(PB0)
#define KEYH _BV(PB1)
#define TONE _BV(PB2)

void init() {
	// keyed when low set high
	PORTB |= KEYL;

	// output setup
	DDRB = KEYL | KEYH | TONE;
}

void key(unsigned long us) {
	PORTB ^= KEYL | KEYH;
	for(; us > 0; us -= BEEP_HALF_PERIOD) {
		PORTB ^= TONE;
		_delay_us(BEEP_HALF_PERIOD);
	}
	PORTB ^= KEYL | KEYH;
}

void sendMessage() {
	int i, j;
	for(i = 0; i < LENGTH; i++) {
		for(j = 0; j < 4; j++) {
			switch((pgm_read_byte(&morseData[i]) >> (6-(j*2))) & 0x03) {
				case 0:
					_delay_us(DIT_TIME * 7);
					break;
				case 1:
					_delay_us(DIT_TIME * 3);
					break;
				case 2:
					key(DIT_TIME);
					_delay_us(DIT_TIME);
					break;
				case 3:
					key(DIT_TIME * 3);
					_delay_us(DIT_TIME);
					break;
			}
		}
	}
}

int main() {

	init();
	while(1) sendMessage();

	return 0; // unreached
}
